#include <iostream>
#include <vector>
#include <RInside.h>
#include <PortfolioPerformanceInput.h>
#include <PortfolioPerformanceOutput.h>
#include <PerformanceCalculator.h>

using namespace std;
using namespace Rcpp;

Rcpp::NumericMatrix createMatrix(const int n)
{
    Rcpp::NumericMatrix M(n,n);
    for (int i = 0; i < n; i++)
    {
        for (int j = 0; j < n; j++)
        {
            M(i,j) = i*10 + j;
        }
    }
    return M;
}

Rcpp::NumericVector createVector(const int n)
{
    Rcpp::NumericVector V(n);
    for (int i = 0; i < n; i++)
    {
        V(i) = i*10;
    }
    return V;
}

int main(int argc, char *argv[])
{

    PortfolioPerformanceInput *ppi;
    ppi = new PortfolioPerformanceInput;

    vector<string> l(1);
    l.push_back("NewAT");

    vector<string> d(5);
    d.push_back("2010-08-06");
    d.push_back("2010-08-13");
    d.push_back("2010-08-20");
    d.push_back("2010-08-27");
    d.push_back("2010-09-03");

    NumericMatrix m(5,3);

    for (unsigned i = 0; i < 5; i++)
    {
        for (unsigned k = 0; k < 3; k++)
        {
            m(i,k) = 100 + k;
        }
    }

    ppi->libraries = l;
    ppi->cashFlows = m;
    ppi->dates = d;
    ppi->benchmark = "BT05";
    ppi->creationDate = "2010-08-06";
    ppi->activationDate = "2010-08-06";
    ppi->finalDate = "2010-09-03";
    ppi->datesBmk = d;
    ppi->indexesBmk = m;
    ppi->start = 100.0;
    ppi->type = "GROSS";
    ppi->weightsBmk = m;

    PerformanceCalculator *pc = new PerformanceCalculator;
    PortfolioPerformanceOutput ppo = pc->calculate(*ppi);

    const int mdim = 4;
    SEXP ans;

    RInside R(argc,argv);

    Rcpp::NumericMatrix M = createMatrix(mdim);
    R["M"] = M;

    Rcpp::NumericVector V = createVector(mdim);
    R["V"] = V;


    string evalstr = "Z <- colSums(M); Z";
    //string evalstr = "Z <- colSums(M); print(Z); Z";

    ans = R.parseEval(evalstr);

    Rcpp::NumericVector v(ans);
    for (int i=0; i<v.size(); i++)
    {
        cout << "In C++ element " << i << " is " << v[i] << endl;
    }

    //evalstr = "K <- sum(V); print(V); print(K); K";
    evalstr = "K <- sum(V); K";

    ans = R.parseEval(evalstr);
    double *value = REAL(ans);

    cout << "Value is: " << *value << endl;
    return 0;
}
